@@ -30,7 +30,7 @@ pub fn read_header<T: Read + Write>(stream: &mut T) -> Vec<u8> {
     buffer
 }
 
-fn handle_client<T: Read + Write>(mut stream: T, root_path: &str, reload: bool) {
+fn handle_client<T: Read + Write>(mut stream: T, root_path: &str, reload: bool, headers: &str) {
     let buffer = read_header(&mut stream);
     let request_string = str::from_utf8(&buffer).unwrap();
 
@@ -58,19 +58,24 @@ fn handle_client<T: Read + Write>(mut stream: T, root_path: &str, reload: bool)
 
     let extension = path.extension().and_then(OsStr::to_str);
 
-    // If no extension is specified assume html
-    let path = if extension == None {
-        path.with_extension("html")
+    let (file_contents, extension) = if extension != None {
+        (fs::read(&path), extension)
     } else {
-        path.to_owned()
+        // If the request has no extension look first for a matching file without an extension
+        if let Ok(file_contents) = fs::read(&path) {
+            println!("WARNING: Serving file without extension: [ {} ] with media type 'application/octet-stream'", &path.to_str().unwrap());
+            (Ok(file_contents), None)
+        } else {
+            // If no file without an extension is found see if there's a file with a ".html" extension
+            // This enables "pretty URLs" without a trailing `/` like: `example.com/blog-post`
+            let file = fs::read(&path.with_extension("html"));
+            (file, Some("html"))
+        }
     };
-    let extension = extension.unwrap_or("html");
-
-    let file_contents = fs::read(&path);
 
     if let Ok(mut file_contents) = file_contents {
-        // Pair the file extension to a mime type.
-        let content_type = extension_to_mime_impl(Some(extension));
+        // Pair the file extension to a media (also known as MIME) type.
+        let content_type = extension_to_mime_impl(extension);
         let mut content_length = file_contents.len();
 
         // Prepare to inject code into HTML if reload is enabled.
@@ -78,14 +83,14 @@ fn handle_client<T: Read + Write>(mut stream: T, root_path: &str, reload: bool)
         let reload_append = include_bytes!("reload.html");
         #[cfg(feature = "reload")]
         {
-            if extension == "html" && reload {
+            if extension == Some("html") && reload {
                 content_length += reload_append.len();
             }
         }
 
         let response = format!(
-            "HTTP/1.1 200 OK\r\nContent-type: {}\r\nContent-Length: {}\r\n\r\n",
-            content_type, content_length
+            "HTTP/1.1 200 OK\r\nContent-type: {}\r\nContent-Length: {}{}\r\n\r\n",
+            content_type, content_length, headers
         );
 
         let mut bytes = response.as_bytes().to_vec();
@@ -95,7 +100,7 @@ fn handle_client<T: Read + Write>(mut stream: T, root_path: &str, reload: bool)
         // Inject code into HTML if reload is enabled
         #[cfg(feature = "reload")]
         {
-            if extension == "html" && reload {
+            if extension == Some("html") && reload {
                 // Insert javascript for reloading
                 stream.write_all(reload_append).unwrap();
             }
@@ -110,9 +115,9 @@ fn handle_client<T: Read + Write>(mut stream: T, root_path: &str, reload: bool)
     }
 }
 
-pub fn run(address: &str, port: u32, path: &str, reload: bool) {
+pub fn run(address: &str, port: u32, path: &str, reload: bool, headers: &str) {
     // Hard coded certificate generated with the following commands:
-    // openssl req -x509 -newkey rsa:1024 -keyout key.pem -out cert.pem -days 36500 -nodes -subj "/"
+    // openssl req -x509 -newkey rsa:2048 -keyout key.pem -out cert.pem -days 36500 -nodes -subj "/"
     // openssl pkcs12 -export -out identity.pfx -inkey key.pem -in cert.pem
     // password for second command: 'debug'
     let bytes = include_bytes!("identity.pfx");
@@ -138,6 +143,7 @@ pub fn run(address: &str, port: u32, path: &str, reload: bool) {
         if let Ok(stream) = stream {
             let acceptor = acceptor.clone();
             let path = path.to_owned();
+            let headers = headers.to_owned();
             thread::spawn(move || {
                 // HTTP requests always begin with a verb like 'GET'.
                 // HTTPS requests begin with a number, so peeking and checking for a number
@@ -151,10 +157,10 @@ pub fn run(address: &str, port: u32, path: &str, reload: bool) {
                 if is_https {
                     // acceptor.accept will block indefinitely if called with an HTTP stream.
                     if let Ok(stream) = acceptor.accept(stream) {
-                        handle_client(stream, &path, reload);
+                        handle_client(stream, &path, reload, &headers);
                     }
                 } else {
-                    handle_client(stream, &path, reload);
+                    handle_client(stream, &path, reload, &headers);
                 }
             });
         }
